// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "BackBufferRenderTarget.hpp"

using namespace Dawn;

BackBufferRenderTarget::BackBufferRenderTarget(RenderManager &renderManager) :
  renderManager(renderManager)
{
}

float_t BackBufferRenderTarget::getWidth() {
  return this->width;
}

float_t BackBufferRenderTarget::getHeight() {
  return this->height;
}

void BackBufferRenderTarget::setSize(float_t width, float_t height) {
  if(this->width == width && this->height == height) return;
  
  // Fixes a new bug that it seems GLFW has introduced.
  if(width == 0) width = 1;
  if(height == 0) height = 1;

  this->width = width;
  this->height = height;
  this->eventRenderTargetResized.invoke(this, width, height);
}

void BackBufferRenderTarget::setClearColor(struct Color color) {
  this->clearColor = color;
}

void BackBufferRenderTarget::clear(flag8_t clearFlags) {
  glClearColor(clearColor.r, clearColor.g, clearColor.b, clearColor.a);
  assertNoGLError();
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  assertNoGLError();
}

void BackBufferRenderTarget::bind() {
  glBindFramebuffer(GL_FRAMEBUFFER, 0);
  assertNoGLError();
  glViewport(0, 0, (GLsizei)this->width, (GLsizei)this->height);
  assertNoGLError();
}